// Copyright © 2023 ab180. All rights reserved.

#include "AirbridgeWebInterface.h"

#include "AirbridgeUnreal.h"
#include "Utils/NumberUtils.h"

//////////////////////////////////////////////////////////////////////////
// AirbridgeWebInterfaceDefault

const FString AirbridgeWebInterfaceDefault::GetScript() { return TEXT(""); }

void AirbridgeWebInterfaceDefault::Handle(const FString& Message)
{
}

//////////////////////////////////////////////////////////////////////////
// AirbridgeWebInterfaceImpl

AirbridgeWebInterfaceImpl* AirbridgeWebInterfaceImpl::Init(
	const FString& InWebToken, const AirbridgePostCommandFunction& InPostCommandFunction
) {
	WebToken = InWebToken;
	PostCommandFunction = InPostCommandFunction;

	return this;
}

const FString AirbridgeWebInterfaceImpl::GetScript()
{
	constexpr int32 JsonSchemaVersion = 4;
	const FString SdkVersion = "1.3.3";

	return FString::Printf(
		TEXT(
			"AirbridgeNative = { };\n"
			"AirbridgeNative.postCommand = function (argument) {\n"
			"    %s\n"
			"};\n"
			"AirbridgeNative.getWebToken = function () {\n"
			"    return \"%s\";\n"
			"};\n"
			"AirbridgeNative.getJsonSchemaVersion = function () {\n"
			"    return %d;\n"
			"};\n"
			"AirbridgeNative.getSdkVersion = function () {\n"
			"    return \"%s\";\n"
			"};\n"
			"AirbridgeNative.setUser = function (payload) {\n"
			"    AirbridgeNative.postCommand(JSON.stringify({\n"
			"        method: \"setUser\"\n,"
			"        payload: payload\n"
			"    }))\n"
			"};\n"
			"AirbridgeNative.clearUser = function () {\n"
			"    AirbridgeNative.postCommand(JSON.stringify({\n"
			"        method: \"clearUser\"\n,"
			"        payload: { }\n"
			"    }))\n"
			"};\n"
			"AirbridgeNative.trackEvent = function (payload) {\n"
			"    AirbridgeNative.postCommand(JSON.stringify({\n"
			"        method: \"trackEvent\"\n,"
			"        payload: payload\n"
			"    }))\n"
			"};\n"
		),
		*PostCommandFunction("argument"),
		*WebToken,
		JsonSchemaVersion,
		*SdkVersion
	);
}

void AirbridgeWebInterfaceImpl::Handle(const FString& Message)
{
	TSharedPtr<FJsonObject> JsonObject = MakeShared<FJsonObject>();
	TSharedPtr<FJsonObject> Payload = MakeShared<FJsonObject>();

	TSharedRef<TJsonReader<>> Reader = TJsonReaderFactory<>::Create(Message);
	if (FJsonSerializer::Deserialize(Reader, JsonObject) && JsonObject.IsValid())
	{
		if (JsonObject->HasField("method"))
		{
			const FString Method = JsonObject->GetStringField("method");

			/** Set User */
			if (Method.Equals("setUser"))
			{
				if (JsonObject->HasField("payload"))
				{
					const FString PayloadString = JsonObject->GetStringField("payload");
					Reader = TJsonReaderFactory<>::Create(PayloadString);
					if (FJsonSerializer::Deserialize(Reader, Payload) && Payload.IsValid())
					{
						// Set User ID
						if (Payload->HasField(TEXT("id")))
						{
							FAirbridgeUnreal::SetUserID(Payload->GetStringField(TEXT("id")));
						}
						// Set User Email
						if (Payload->HasField(TEXT("email")))
						{
							FAirbridgeUnreal::SetUserEmail(Payload->GetStringField(TEXT("email")));
						}
						// Set User Phone Number
						if (Payload->HasField(TEXT("phone")))
						{
							FAirbridgeUnreal::SetUserPhone(Payload->GetStringField(TEXT("phone")));
						}

						// Set User Alias
						if (Payload->HasField(TEXT("alias")))
						{
							const TSharedPtr<FJsonObject> Alias = Payload->GetObjectField(TEXT("alias"));
							if (Alias.IsValid())
							{
								for (const auto& Element : Alias->Values)
								{
									FString FieldName = Element.Key;
									TSharedPtr<FJsonValue> FieldValue = Element.Value;
									if (FieldValue.IsValid() && FieldValue->Type == EJson::String)
									{
										FAirbridgeUnreal::SetUserAlias(FieldName, FieldValue->AsString());
									}
								}
							}
						}

						// Set User Attributes
						if (Payload->HasField(TEXT("attributes")))
						{
							const TSharedPtr<FJsonObject> Attributes = Payload->GetObjectField(TEXT("attributes"));
							if (Attributes.IsValid())
							{
								for (const auto& Element : Attributes->Values)
								{
									FString FieldName = Element.Key;
									TSharedPtr<FJsonValue> FieldValue = Element.Value;
									if (FieldValue.IsValid())
									{
										switch (FieldValue->Type)
										{
										case EJson::String:
											FAirbridgeUnreal::SetUserAttribute(FieldName, FieldValue->AsString());
											break;
										case EJson::Number:
											if (NumberUtils::IsConvertibleToInt64(FieldValue->AsNumber()))
											{
												if (NumberUtils::IsConvertibleToInt32(FieldValue->AsNumber()))
													FAirbridgeUnreal::SetUserAttribute(
														FieldName, static_cast<int32>(FieldValue->AsNumber()));
												else
													FAirbridgeUnreal::SetUserAttribute(
														FieldName, static_cast<int64>(FieldValue->AsNumber()));
											}
											else
											{
												// Convert a double to a float
												// be aware of the potential loss of precision when converting between data types
												FAirbridgeUnreal::SetUserAttribute(
													FieldName, static_cast<float>(FieldValue->AsNumber()));
											}
											break;
										case EJson::Boolean:
											FAirbridgeUnreal::SetUserAttribute(FieldName, FieldValue->AsBool());
											break;
										default:
											break;
										}
									}
								}
							}
						}
					}
				}
			}
			/** Clear User */
			else if (Method.Equals(TEXT("clearUser")))
			{
				FAirbridgeUnreal::ClearUser();
			}
			/** Track Event */
			else if (Method.Equals(TEXT("trackEvent")))
			{
				if (JsonObject->HasField(TEXT("payload")))
				{
					const FString PayloadString = JsonObject->GetStringField(TEXT("payload"));
					Reader = TJsonReaderFactory<>::Create(PayloadString);
					if (FJsonSerializer::Deserialize(Reader, Payload) && Payload.IsValid() &&
						Payload->HasField(TEXT("category"))) // Event Category Is Required
					{
						UAirbridgeEventOption* AirbridgeEventOption = UAirbridgeEventOption::CreateObject();
						// Set Action
						if (Payload->HasField(TEXT("action")))
						{
							AirbridgeEventOption->SetAction(Payload->GetStringField(TEXT("action")));
						}
						// Set Label
						if (Payload->HasField(TEXT("label")))
						{
							AirbridgeEventOption->SetLabel(Payload->GetStringField(TEXT("label")));
						}
						// Set Value
						if (Payload->HasField(TEXT("value")))
						{
							AirbridgeEventOption->SetValue(Payload->GetNumberField(TEXT("value")));
						}
						// Set Semantic Attributes
						if (Payload->HasField(TEXT("semantic_attributes")))
						{
							const TSharedPtr<FJsonObject>* SemanticAttributes;
							if (Payload->TryGetObjectField(TEXT("semantic_attributes"), /*out*/ SemanticAttributes)
								&& SemanticAttributes->IsValid())
							{
								AirbridgeEventOption->SetSemanticAttributes(UAirbridgeMap::CreateObject(*SemanticAttributes));
							}
						}
						// Set Custom Attributes
						if (Payload->HasField(TEXT("custom_attributes")))
						{
							const TSharedPtr<FJsonObject>* CustomAttributes;
							if (Payload->TryGetObjectField(TEXT("custom_attributes"), /*out*/ CustomAttributes)
								&& CustomAttributes->IsValid())
							{
								AirbridgeEventOption->SetCustomAttributes(UAirbridgeMap::CreateObject(*CustomAttributes));
							}
						}
						FAirbridgeUnreal::TrackEvent(Payload->GetStringField(TEXT("category")), AirbridgeEventOption);
					}
				}
			}
		}
	}
}
